package geniusweb.references;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import tudelft.utilities.junit.GeneralTests;

public class PartiWithParamsTest extends GeneralTests<PartyWithParameters> {

	private PartyRef party1, party2;
	private Parameters param1, param2;
	private PartyWithParameters party1param1, party1param1a;
	private PartyWithParameters party1param2;
	private final String serialized = "{\"partyref\":\"ws:party1\",\"parameters\":{\"a\":1}}";

	@Before
	public void before() throws URISyntaxException {
		party1 = new PartyRef(new URI("ws:party1"));
		party2 = new PartyRef(new URI("ws:party2"));
		param1 = new Parameters();
		param2 = new Parameters();
		param2.put("a", 1);
		party1param1 = new PartyWithParameters(party1, param1);
		party1param1a = new PartyWithParameters(party1, param1);
		party1param2 = new PartyWithParameters(party1, param2);
	}

	@Override
	public List<List<PartyWithParameters>> getGeneralTestData() {
		return Arrays.asList(
				Arrays.asList(party1param1, party1param1a, party1param1a),
				Arrays.asList(party1param2));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("PartyRef.ws:party1.",
				"PartyRef.ws:party1.\\{a=1\\}");
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(party1param2);
		System.out.println(json);
		assertEquals(serialized, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		PartyWithParameters p = jackson.readValue(serialized,
				PartyWithParameters.class);
		System.out.println(p);
		assertEquals(party1param2, p);
	}

}
